#include "Expression.h"

ExpressionNode::ExpressionNode(Symbol* s, int childcount){
  this->symbol = s->getPointer();
  this->childcount = childcount;
  parent = NULL;
  children = new ExpressionNode* [childcount];
  for(int i=0; i < childcount; i++)
    children[i] = NULL;
}
ExpressionNode::~ExpressionNode(){
  symbol->removePointer();
}

ExpressionNode* ExpressionNode::clone(){
  ExpressionNode* ret = new ExpressionNode(this->symbol, this->childcount);
  for(int i=0; i < this->childcount; i++){
    ExpressionNode* child = getChild(i)->clone();
    ret->setChild(i, child);
    child->parent = ret;
  }
  
  return ret;
}

void ExpressionNode::setChild(int i, ExpressionNode* node){
  if(i >= 0 && i < childcount)
    children[i] = node;
}
ExpressionNode* ExpressionNode::getChild(int i){
  if(i >= 0 && i < childcount)
    return children[i];
  else
    return NULL;
}
int ExpressionNode::getChildCount(){
  return childcount;
}

Expression::Expression(int linenumber){
  lineno = linenumber;
  root = NULL;
}
Expression::~Expression(){
  destroy(root);
}

Expression* Expression::clone(){
  Expression* exp = new Expression(getLineNumber());

  ExpressionNode* root = getRoot();
  if(root != NULL)
    root = root->clone();
  exp->setRoot(root);

  return exp;
}


int Expression::getLineNumber(){
  return lineno;
}

ExpressionNode* Expression::getRoot(){
  return root;
}
void Expression::setRoot(ExpressionNode* root){
  this->root = root;
}

ExpressionNode* Expression::createNode(Symbol* s, int childcount){
  return new ExpressionNode(s, childcount);
}

void Expression::destroy(ExpressionNode* node){
  if(node == NULL)
    return;
  
  for(int i=0; i < node->getChildCount(); i++){
    if(node->getChild(i) != NULL)
      destroy(node->getChild(i));
  }
  if(node->parent != NULL){
    for(int i=0; i < node->parent->getChildCount(); i++){
      if(node->parent->getChild(i) == node)
	node->parent->setChild(i, NULL);
    }
  }
  delete node;
}
ExpressionNode* Expression::insert(ExpressionNode* node, ExpressionNode* parent, int branch){
  ExpressionNode* oldchild = parent->getChild(branch);
  if(oldchild != NULL){
    if(add(oldchild, node) == false)
      destroy(oldchild);
  }
  node->parent = parent;
  parent->setChild(branch, node);
  return node;
}
bool Expression::add(ExpressionNode* node, ExpressionNode* parent){
  for(int i=0; i < parent->getChildCount(); i++){
    if(parent->getChild(i) == NULL){
      parent->setChild(i, node);
      node->parent = parent;
      return true;
    }
  }
  return false;
}

string Expression::toString(){
  string s;
  append(&s, getRoot());
  return s;
}
void Expression::append(string* s, ExpressionNode* node){
  switch(node->getChildCount()){
  case 0:
    s->append(node->symbol->toString());
    break;
			
  case 1:
    if(node->symbol->isType(Symbol::LPAR)){
      s->append("(");
      append(s, node->getChild(0));
      s->append(")");
    }else if(node->symbol->isType(Symbol::PREFIX)){
      s->append(node->symbol->toString());
      append(s, node->getChild(0));
    }else{
      append(s, node->getChild(0));
      s->append(node->symbol->toString());
    }
    break;
			
  case 2:
    append(s, node->getChild(0));
    s->append(node->symbol->toString());
    append(s, node->getChild(1));
    break;
  }
}
