#include "MainLib.h"
#include <iostream>

void MainLib::load(SymbolTable * symbol_table){
  SymbolTable* hardwired = symbol_table->getHardwired();

  hardwired->set(new Symbol(Symbol::LPAR, "("));
  hardwired->set(new Symbol(Symbol::RPAR, ")"));
  hardwired->set(new Symbol(Symbol::LBRACKET, "{"));
  hardwired->set(new Symbol(Symbol::RBRACKET, "}"));
  hardwired->set(new Symbol(Symbol::DEF, "def"));
  hardwired->set(new Symbol(Symbol::UNARY_DEF, "unary"));
  hardwired->set(new Symbol(Symbol::BINARY_DEF, "binary"));
  hardwired->set(new Symbol(Symbol::PREFIX_DEF, "prefix"));
	
  symbol_table->set(new Function(Symbol::UNARY, "abs", 		&Abs));
  symbol_table->set(new Function(Symbol::UNARY, "asDouble", 	&AsDouble));
  symbol_table->set(new Function(Symbol::UNARY, "asInteger", 	&AsInteger));
  symbol_table->set(new Function(Symbol::UNARY, "count", 	&Count));
  symbol_table->set(new Function(Symbol::UNARY, "exp", 		&Exp));
  symbol_table->set(new Function(Symbol::UNARY, "inv", 		&Inv));
  symbol_table->set(new Function(Symbol::UNARY, "len", 		&Len));
  symbol_table->set(new Function(Symbol::UNARY, "ln", 		&Ln));
  symbol_table->set(new Function(Symbol::UNARY, "log", 		&Log));
  symbol_table->set(new Function(Symbol::UNARY, "neg", 		&Neg));
  symbol_table->set(new Function(Symbol::UNARY, "rank", 	&Rank));
  symbol_table->set(new Function(Symbol::UNARY, "ravel", 	&Ravel));
  symbol_table->set(new Function(Symbol::UNARY, "floor", 	&Floor));
  symbol_table->set(new Function(Symbol::UNARY, "ceil", 	&Ceil));
  symbol_table->set(new Function(Symbol::UNARY, "shape", 	&Shape));
  symbol_table->set(new Function(Symbol::UNARY, "trans", 	&Trans));
	
  symbol_table->set(new Function(Symbol::BINARY, "/", 	&Divide));
  symbol_table->set(new Function(Symbol::BINARY, "*", 	&Multiply));
  symbol_table->set(new Function((Symbol::SymbolType)(Symbol::BINARY + Symbol::PREFIX), "-", &Minus));
  symbol_table->set(new Function((Symbol::SymbolType)(Symbol::BINARY + Symbol::PREFIX), "+", &Plus));
	
  symbol_table->set(new Function(Symbol::BINARY, "cat", 	&Cat));
  symbol_table->set(new Function(Symbol::BINARY, "max", 	&Max));
  symbol_table->set(new Function(Symbol::BINARY, "min", 	&Min));
  symbol_table->set(new Function(Symbol::BINARY, "pow", 	&Pow));
  symbol_table->set(new Function(Symbol::BINARY, "reshape", &Reshape));
  symbol_table->set(new Function(Symbol::BINARY, "at", 	&At));
		
  symbol_table->set(new Function(Symbol::BINARY, ">=", 	&GtOrEq));
  symbol_table->set(new Function(Symbol::BINARY, "<=", 	&LtOrEq));
  symbol_table->set(new Function(Symbol::BINARY, "<", 	&Less));
  symbol_table->set(new Function(Symbol::BINARY, ">", 	&Greater));
  symbol_table->set(new Function(Symbol::BINARY, "==", 	&Equals));
  symbol_table->set(new Function(Symbol::BINARY, "<>", 	&Unequal));
	
  symbol_table->set(new Function(Symbol::BINARY, "=", 	&Assign));
}

Symbol* MainLib::Abs(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Abs(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar( (val < 0) ? -val : val);
  }
}
Symbol* MainLib::AsDouble(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::AsDouble(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar(val);
  }
}
Symbol* MainLib::AsInteger(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::AsInteger(arg, 1));
    }
    return dest;
  }else{
    int val = (int)((Scalar*)c)->getValue();
    return new Scalar(val);
  }
}

Symbol* MainLib::Count(Symbol* args[], int argc){
  Vector* dest = new Vector();
	
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::SCALAR){
    for(int i=0; i < ((Scalar*)c)->getValue(); i++)
      dest->add(new Scalar(i+1));
  }
	
  return dest;
}
Symbol* MainLib::Exp(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Exp(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar(exp(val));
  }
}
Symbol* MainLib::Inv(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Inv(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar( 1 / val);
  }
}
Symbol* MainLib::Len(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
  if(c->ctype == Constant::VECTOR)
    return new Scalar(((Vector*)c)->size());
  else
    return new Scalar(1.0);
}
Symbol* MainLib::Ln(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Ln(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar(log(val));
  }
}
Symbol* MainLib::Log(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Log(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar(log10(val));
  }
}
Symbol* MainLib::Neg(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Neg(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar( -val );
  }
}
Symbol* MainLib::Rank(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  Scalar* s;
  if(c->ctype == Constant::VECTOR){
    Symbol* arg[] = { ((Vector*)c)->get(0) };
    s = (Scalar*)MainLib::Rank(arg, 1);
    s->setValue(s->getValue() + 1);
  }else
    s = new Scalar(0.0);
		
  return s;
}
Symbol* MainLib::Ravel(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
		
    for(int i=0; i < src->size(); i++){
      Constant* vc = src->get(i)->getConstant();
			
      if(vc->ctype == Constant::VECTOR){
	Vector* vv = (Vector*)vc;
				
	for(int i=0; i < vv->size(); i++)
	  dest->add(vv->get(i));
      }else
	dest->add(vc);
    }
    return dest;
  }
  return c;
}
Symbol* MainLib::Floor(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Floor(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar( floor(val) );
  }
}
Symbol* MainLib::Ceil(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
    Vector* src = (Vector*)c;
    Vector* dest = new Vector();
    Symbol* arg[1];
    for(int i = 0; i < src->size(); i++){
      arg[0] = src->get(i);
      dest->add(MainLib::Ceil(arg, 1));
    }
    return dest;
  }else{
    double val = ((Scalar*)c)->getValue();
    return new Scalar( ceil(val) );
  }
}
Symbol* MainLib::Shape(Symbol* args[], int argc){
  Constant* c = args[0]->getConstant();
	
  if(c->ctype == Constant::VECTOR){
		
    Constant* cc = ((Vector*)c)->get(0)->getConstant();
    Vector* dest;
    if(cc->ctype == Constant::VECTOR){
      Symbol* arg[] = { cc };
      dest = (Vector*)MainLib::Shape(arg, 1);
      dest->insert(new Scalar(((Vector*)c)->size()), 0);
    }else
      dest = new Vector(new Scalar(((Vector*)c)->size()));
    return dest;
  }else
    return c;
}
Symbol* MainLib::Trans(Symbol* args[], int argc){
  return new Scalar(0.0);
}

Symbol* MainLib::Plus(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(argc == 1)
    return c[0]->copy();
	
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR)
    return new Scalar(((Scalar*)c[0])->getValue() + ((Scalar*)c[1])->getValue());
  else
    return MainLib::cycle(c, &MainLib::Plus);
}
Symbol* MainLib::Minus(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(argc == 1){
    switch(c[0]->ctype){
    case Constant::SCALAR:
      return new Scalar(0 - ((Scalar*)c[0])->getValue());
    case Constant::VECTOR:
      Vector* dest = new Vector();
      for(int i=0; i < ((Vector*)c[0])->size(); i++){
	Symbol* args[] = { ((Vector*)c[0])->get(i) };
	dest->add(MainLib::Minus(args, 1));
      }
      return dest;
    }
    return c[0]->copy();
  }
	
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR)
    return new Scalar(((Scalar*)c[0])->getValue() - ((Scalar*)c[1])->getValue());
  else
    return MainLib::cycle(c, &MainLib::Minus);
}
Symbol* MainLib::Multiply(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR)
    return new Scalar(((Scalar*)c[0])->getValue() * ((Scalar*)c[1])->getValue());
  else
    return MainLib::cycle(c, &MainLib::Multiply);
}
Symbol* MainLib::Divide(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR)
    return new Scalar(((Scalar*)c[0])->getValue() / ((Scalar*)c[1])->getValue());
  else
    return MainLib::cycle(c, &MainLib::Divide);
}
Symbol* MainLib::Assign(Symbol* args[], int argc){
  if(args[0]->type == Symbol::ID)
    ((Variable*)args[0])->setValue(args[1]->getConstant());
  else {
    throw new RuntimeException("For assign('=') to work the left operand needs to be a variable.");
  }
  return args[0]->getConstant();
}

Symbol* MainLib::Equals(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() == ((Scalar*)c[1])->getValue()) ? 1 : 0;
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::Equals);
}
Symbol* MainLib::GtOrEq(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() >= ((Scalar*)c[1])->getValue()) ? 1 : 0;
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::GtOrEq);
}
Symbol* MainLib::LtOrEq(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)	
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() <= ((Scalar*)c[1])->getValue()) ? 1 : 0;
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::LtOrEq);
}
Symbol* MainLib::Unequal(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() != ((Scalar*)c[1])->getValue()) ? 1 : 0;
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::Unequal);
}
Symbol* MainLib::Less(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() < ((Scalar*)c[1])->getValue()) ? 1 : 0;
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::Less);
}
Symbol* MainLib::Greater(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() > ((Scalar*)c[1])->getValue()) ? 1 : 0;
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::Greater);
}

Symbol* MainLib::Cat(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  Vector* dest = new Vector();
	
  for(int i=0; i < 2; i++){
		
    if(c[i]->ctype == Constant::VECTOR){
      Vector* v = (Vector*)c[i];
      for(int j=0; j < v->size(); j++)
	dest->add(v->get(j)->getConstant()->copy());
    }else
      dest->add(c[i]->copy());
  }
  return dest;
}
Symbol* MainLib::Max(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() > ((Scalar*)c[1])->getValue()) ? ((Scalar*)c[0])->getValue() : ((Scalar*)c[1])->getValue();
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::Max);
}
Symbol* MainLib::Min(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = (((Scalar*)c[0])->getValue() < ((Scalar*)c[1])->getValue()) ? ((Scalar*)c[0])->getValue() : ((Scalar*)c[1])->getValue();
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::Min);
}
Symbol* MainLib::Pow(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::SCALAR && c[1]->ctype == Constant::SCALAR){
    double x = pow(((Scalar*)c[0])->getValue(), ((Scalar*)c[1])->getValue());
    return new Scalar(x);
  }else
    return MainLib::cycle(c, &MainLib::Pow);
}

Symbol* MainLib::Reshape(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();

  Vector* symbol = (c[0]->ctype == Constant::VECTOR) ? NULL :  new Vector(c[0]);
  Vector* dims = (c[1]->ctype == Constant::VECTOR) ? NULL :  new Vector(c[1]);

  int cycle = 0;
  Vector* ret = MainLib::Reshape((symbol == NULL) ? (Vector*)c[0] : symbol, 
				 (dims == NULL) ? (Vector*)c[1] : dims, 
				 0,
				 cycle);

  if(symbol != NULL)
    delete symbol;
  if(dims != NULL)
    delete dims;
  return ret;
	
}
Vector* MainLib::Reshape(Vector* symbols, Vector* dims, int scope, int& cycle){
  int len = (int)((Scalar*)dims->get(scope))->getValue();
  Vector* v = new Vector();
  scope++;
	
  for(int i=0; i < len; i++){
    if(scope < dims->size())
      v->add(MainLib::Reshape(symbols, dims, scope, cycle));
    else{
      v->add(symbols->get(cycle)->getConstant()->copy());
      cycle = (cycle+1)%symbols->size();
    }
  }
  return v;
}
Symbol* MainLib::At(Symbol* args[], int argc){
  Constant* c[argc];
  for(int i=0; i < argc; i++)
    c[i] = args[i]->getConstant();
		
  if(c[0]->ctype == Constant::VECTOR && c[1]->ctype == Constant::SCALAR){
    int i = (int)((Scalar*)c[1])->getValue();
    i--;
    Vector* v = (Vector*)c[0];
    if(i >= 0 && i < v->size()){
      Symbol* s = v->get(i);
      if(s != NULL)
	return s->getConstant();
    }
  }
  return new Scalar(0.0);
}


Vector * MainLib::cycle(Constant* c[], Symbol* (*f) (Symbol* args[], int argc)){
  Vector * ret = new Vector();
	
  bool run[] = {true, true};
  Symbol* s[2];
  int x = 0;
  while(true){
    for(int i=0; i < 2; i++){
      if(c[i]->ctype == Constant::VECTOR){
	if(x < ((Vector*)c[i])->size())
	  s[i] = ((Vector*)c[i])->get(x);
	else	
	  run[i] = false;
      }else
	s[i] = c[i];
    }
    if(run[0] == false || run[1] == false){
      if(run[0] == true || run[1] == true) {
	for(int i=0; i < 2; i++){
	  if(run[i] == true && c[i]->ctype == Constant::VECTOR){
	    for(; x < ((Vector*)c[i])->size(); x++)
	      ret->add(((Vector*)c[i])->get(x));
							
	    break;
	  }
	}
      }
      break;
    }else{
      ret->add(f(s, 2));
      x++;
    }
  }
	
  return ret;
}

