#ifndef __MainLib_h__
#define __MainLib_h__

#include "Symbol.h"
#include "SymbolTable.h"
#include "RuntimeException.h"
#include <exception>
#include <stdlib.h>
#include <math.h>

class MainLib{
private:
  static Vector * cycle(Constant * c[], Symbol* (*f) (Symbol* args[], int argc));
  static int compare(Symbol* args[]);

public:
  static void load(SymbolTable * symbol_table);
	
  //unaire operatoren
  //absolute waarde van args[0]
  static Symbol* Abs(Symbol* args[], int argc);
  //int args[0] wordt een double
  static Symbol* AsDouble(Symbol* args[], int argc);
  //double args[0] wordt een int
  static Symbol* AsInteger(Symbol* args[], int argc);
  //1 2 3 ... args[0]
  static Symbol* Count(Symbol* args[], int argc);
  //pow(e, args[0])
  static Symbol* Exp(Symbol* args[], int argc);
  //1 / args[0]
  static Symbol* Inv(Symbol* args[], int argc);
  //aantal elementen in args[0]
  static Symbol* Len(Symbol* args[], int argc);
  //natuurlijk logaritme van args[0]
  static Symbol* Ln(Symbol* args[], int argc);
  //logaritme basis 10 van args[0]
  static Symbol* Log(Symbol* args[], int argc);
  //- args[0]
  static Symbol* Neg(Symbol* args[], int argc);
  // 1 rank = 0, 1 2 3 rank = 1, (1 2)(1 2) rank = 2, enz
  static Symbol* Rank(Symbol* args[], int argc);
  //maakt van args[0] een 1 dimentionale vector
  static Symbol* Ravel(Symbol* args[], int argc);
  //maak int(rond af naar beneden)
  static Symbol* Floor(Symbol* args[], int argc);
  //maak int(rond af naar boven)
  static Symbol* Ceil(Symbol* args[], int argc);
  //geeft dimenties weer
  static Symbol* Shape(Symbol* args[], int argc);
  static Symbol* Trans(Symbol* args[], int argc);
		
  //binaire operatoren
  //args[0] + args[1]
  static Symbol* Plus(Symbol* args[], int argc);
  //args[0] - args[1]
  static Symbol* Minus(Symbol* args[], int argc);
  //args[0] * args[1]
  static Symbol* Multiply(Symbol* args[], int argc);
  //args[0] / args[1]
  static Symbol* Divide(Symbol* args[], int argc);
  //args[0] = args[1]
  static Symbol* Assign(Symbol* args[], int argc);

  //args[0] == args[1]
  static Symbol* Equals(Symbol* args[], int argc);
  //args[0] >= args[1]
  static Symbol* GtOrEq(Symbol* args[], int argc);
  //args[0] <= args[1]
  static Symbol* LtOrEq(Symbol* args[], int argc);
  //args[0] <> args[1]
  static Symbol* Unequal(Symbol* args[], int argc);
  //args[0] < args[1]
  static Symbol* Less(Symbol* args[], int argc);
  //args[0] > args[1]
  static Symbol* Greater(Symbol* args[], int argc);

  //args[0] en args[1] gecombineerd in 1 vector
  static Symbol* Cat(Symbol* args[], int argc);
  //(args[0] > args[1]) ? args[0] : args[1]
  static Symbol* Max(Symbol* args[], int argc);
  //(args[0] < args[1]) ? args[0] : args[1]
  static Symbol* Min(Symbol* args[], int argc);
  //pow(args[0], args[1])
  static Symbol* Pow(Symbol* args[], int argc);
  static Symbol* Reshape(Symbol* args[], int argc);
  //element number args[1] in vector args[0]
  static Symbol* At(Symbol* args[], int argc);
		
  static Vector* Reshape(Vector* symbols, Vector* dims, int scope, int& cycle);
};

#endif
