#ifndef __Reader_h__
#define __Reader_h__

/**
 * Interface for providing text input to the interpreter.
 * 
 */
class Reader{
 public:
  virtual ~Reader(){}

  /**
   * Read 1 char store it in chr
   *
   * @return true if read was successful or false if there is no more data left.
   * @throws IOException   if an error occured while reading the character.
   */
  virtual bool read(char& chr)=0;

  /**
   * Read len char's and store in buffer.
   *
   * @return the number of characters read.
   * @throws IOException   if an error occured while reading the character.
   */
  virtual unsigned int read(char * buffer, unsigned int len)=0;

  /**
   * Read characters until a newline character is found or until len characters
   * are read.
   *
   * @return the number of characters read.
   * @throws IOException   if an error occured while reading the character.
   */
  virtual unsigned int readLine(char * buffer, unsigned int len)=0;

  /**
   * Close the reader and free any resources used by the reader.
   */
  virtual void close()=0;
};

#endif
