#ifndef __Symbol_h__
#define __Symbol_h__

#include <string>
#include <vector>
#include <exception>
#include <sstream>
#include <stdlib.h>

using namespace std;

/*		Symbol, Constant, Scalar, Vector, Function, Variable
		Worden gebruikt om tokens mee te identificeren. Scalar heeft bevoorbeeld een double 
		member om een scalar in op te slaan. Vector heeft een vector. Function bevat een 
		pointer naar een functie. Variable heeft een pointer naar een Constant(Scalar/Vector).
	
*/

class Constant;
class Symbol{
 public:
  enum SymbolType { UNARY = 1, BINARY = 2, PREFIX = 4, CONSTANT = 8, ID = 9, 
		    LPAR = 10, RPAR = 11, EOL = 12, EOS = 13,
		    LBRACKET = 14, RBRACKET = 15, DEF = 16, 
		    UNARY_DEF = 17, BINARY_DEF = 18, PREFIX_DEF = 19 } type;
		
  string token;
  int pointercount;
		
  Symbol();
  Symbol(SymbolType type, const char* token);
  bool isType(SymbolType type);

  Symbol* getPointer();
  void removePointer();
		
  virtual ~Symbol(){}
  virtual Constant* getConstant();
  virtual const char* toString();
			
  static Symbol eol, eos;
};



class Constant: public Symbol{
 public:
  enum ConstantType{SCALAR, VECTOR};
  int ctype;
		
  Constant(const char* token);
  Constant* getConstant();
  virtual Constant* copy()=0;
};

class Scalar: public Constant{
 private:
  double val;
 public:
  Scalar(const char* token);
  Scalar(double d);
  void setValue(double value);
  double getValue();
  Constant* copy();
};
class Vector: public Constant{
 private:
  vector<Symbol*> list;
 public:
  Vector();
  Vector(Symbol* s);
  Vector(const char* token);
  ~Vector();

  Symbol* add(Symbol * s);
  void insert(Symbol* s, int i);
  Symbol * get(int i);
  Symbol* remove(int i);
  int size();
  Constant* copy();
  const char* toString();
};

class Function: public Symbol{
 private:
  Symbol** args;
  Symbol * (*func)(Symbol* arg[], int argc);
 public:
  Function(SymbolType type, const char* token, Symbol * (*func)(Symbol* arg[], int argc));
  virtual Symbol * call(Symbol* args[], int argc);
};


class Variable: public Symbol{
 public:
  Constant * value;
  Variable(const char* token);
  ~Variable();
  Constant * getConstant();
  void setValue(Constant * val);
};

class Expression;
#include "Expression.h"
#include "Executrix.h"

class CustomFunction: public Function{
 private:
  Expression** expression;
  int exp_count;
  Variable** args;
  int arg_count;
 public:
  CustomFunction(SymbolType type, const char* token, Expression* expression[], int count, Variable* args[], int argc);
  ~CustomFunction();
  Symbol* call(Symbol* args[], int argc);
};

#endif
