#include "SymbolTable.h"
#include "Expression.h"

SymbolTable::SymbolTable(){
  mapsize = 255;
  map = new List*[mapsize];
  for(int i=0; i < mapsize; i++)
    map[i] = NULL;    
  super = NULL;
  hardwired = NULL;
}

SymbolTable::SymbolTable(int size){
  mapsize = size;
  map = new List*[mapsize];
  for(int i=0; i < mapsize; i++)
    map[i] = NULL;    
  super = NULL;
  hardwired = NULL;
}


SymbolTable::~SymbolTable(){
  for(int i=0; i < mapsize; i++){
    List* l = map[i];
    if(l != NULL)
      delete l;
  }
  super = NULL;
}

SymbolTable* SymbolTable::getSuperScope(){
  return super;
}
void SymbolTable::setSuperScope(SymbolTable* table){
  this->super = table;
}
SymbolTable* SymbolTable::getHardwired(){
  return hardwired;
}
void SymbolTable::setHardwired(SymbolTable* table){
  this->hardwired = table;
}

int SymbolTable::hash(string s){
  unsigned int h = 0, g;
  for (string::size_type i=0; i < s.size(); i++) {
    h = (h << 4) + s.at(i);
    if (g = h & 0xf0000000)
      h = h ^ (g >> 24);
  }
  return h % mapsize;
}

Symbol * SymbolTable::set(Symbol* symbol) {
  if(hardwired != NULL && hardwired->exists(symbol->token))
    throw new CannotAssignSymbolException(symbol->token);

  int h = hash(symbol->token);
	
  if(map[h] == NULL)
    map[h] = new List();

  map[h]->insert(symbol);

  if(symbol->isType(Symbol::BINARY))
    bin_precedence_list.push_back(symbol);
  return symbol;
}

Symbol * SymbolTable::get(string name){
  if(hardwired != NULL){
    Symbol* s = hardwired->get(name);
    if(s != NULL)
      return s;
  }

  int h = hash(name);
  List* list = map[h];

  if(list != NULL)
    return list->get(name);
  else if(super != NULL)
    return super->get(name);
  else
    return NULL;
}

bool SymbolTable::exists(string name){
  return (get(name) != NULL);
}

void SymbolTable::remove(string name){
  int h = hash(name);
  List* list = map[h];
  if(list != NULL)
    list->remove(name);
}

void SymbolTable::clear(){
  for(int i=0; i < mapsize; i++){
    List* l = map[i];
    if(l != NULL){
      l->clear();
      delete l;
      map[i] = NULL;
    }
  }
}

bool SymbolTable::hasHigherPrecedence(Symbol* symbol, Symbol* then){
  for(vector<Symbol*>::size_type i=0; i < bin_precedence_list.size(); i++){
    if(bin_precedence_list[i] == symbol)
      return true;
    else if(bin_precedence_list[i] == then)
      return false;
  }

  if(super != NULL)
    return super->hasHigherPrecedence(symbol, then);
  else
    return false;
}
void SymbolTable::print(ostream& stream){
  for(int i=0; i < mapsize; i++){
    List* l = map[i];
    if(l != NULL){
      l->print(stream);
      stream << endl;
    }
  }
}
