#ifndef __SymbolTable_h__
#define __SymbolTable_h__

#include <vector>
#include <string>
#include <iostream>
#include "List.h"
#include "Symbol.h"

using namespace std;

/*
  SymbolTable
  De SymbolTable is een HashMap. Met set() kunnen symbolen kunnen worden toegevoegd, en met 
  get() kunnen symbolen gevonden worden doormiddel van een string. Symbol* objecten hebben
  een token. Daarvan wordt met de hash functie een waarde berekend en die waarde geeft aan
  op welke plek in de array van List* objecten het Symbol* object wordt ingevoegd. 
  Er wordt voor binaire operatoren een lijst bij gehouden die aangeeft hoe belangrijk een 
  binaire operator is( * wordt eerder uitgevoerd dan + en is dus belangrijker). Operatoren
  die het eerst worden toegevoegd zijn belangrijker dan operatoren die later worden 
  toegevoegd.
*/

class CannotAssignSymbolException: public exception{
 public:
  string err;
	
  CannotAssignSymbolException(string& found){
    err.append("Symbol ");
    err.append(found);
    err.append(" is hardwired and can not be overwritten");
  }

  ~CannotAssignSymbolException() throw () {};
	
  virtual const char* what() const throw(){
    return err.c_str();
  }
};


class SymbolTable{
 public:
		
  SymbolTable();
  SymbolTable(int size);
  SymbolTable(SymbolTable* super);
  ~SymbolTable();

  SymbolTable* getSuperScope();
  void setSuperScope(SymbolTable* table);
  SymbolTable* getHardwired();
  void setHardwired(SymbolTable* table);
		
  Symbol* set(Symbol* symbol);
  Symbol* get(string name);
  void remove(string name);
  bool exists(string name);
  bool hasHigherPrecedence(Symbol* name, Symbol* other);
  void print(ostream& stream);
  void clear();
			
 private:
  int mapsize;	
  List** map;
  SymbolTable* super, *hardwired;
		
  int hash(string s);
				
  vector<Symbol*> bin_precedence_list;
};

#endif
