#include "Symbolizer.h"

Symbolizer::Symbolizer(SymbolTable* symbols, Reader * reader): Tokenizer(reader){
  symbol_table = symbols;
  unseparatedNumber = false;
  readNextToken();
}

void Symbolizer::setSymbolTable(SymbolTable* symboltable){
  this->symbol_table = symboltable;
}

SymbolTable* Symbolizer::getSymbolTable(){
  return symbol_table;
}

Symbol* Symbolizer::readNextSymbol() throw(InvalidTokenException*){
  if(ident == &Symbol::eos)
    return ident;
  else if(ident == &Symbol::eol)
    readNextToken();
  
  Symbol * match;	
  switch(getTokenType()){
  case Tokenizer::NUMBER:
    ident = symbolizeNumber();
    
    while(readNextToken() == Tokenizer::NUMBER){
      if(((Constant*)ident)->ctype == Constant::SCALAR)
	ident = new Vector(ident);
      ((Vector*)ident)->add(symbolizeNumber());
    }
    break;
    
  case Tokenizer::WORD:
    {
      string s;
      bool end = false;

      // Read more tokens until a match is found in the symbol
      // table or the next token is not a word or number.
      // 'x5' should be reckognized as an id, not an id and a number.
      while(!end){
	s.append(*getToken());
	match = symbol_table->get(s);
	if(match != NULL && !match->isType(Symbol::ID))
	  end = true;
	
	if(lastReadType == Tokenizer::WHITESPACE)
	  end = true;
	
	readNextToken();
	
	if(getTokenType() != NUMBER && getTokenType() != WORD)
	  end = true;
      }
				
      if(match == NULL){
	match = new Variable(s.c_str());
	symbol_table->set(match);
      }
      if(unseparatedNumber && match->isType(Symbol::ID)){
	unseparatedNumber = false;
	throw new InvalidTokenException(string(ident->token).append(s));
      }
      unseparatedNumber = false;
      ident = match;
    }
    break;
    
  case Tokenizer::UNKOWN:
    
    unseparatedNumber = false;
    
    match = getMatch(getToken());
    if(match != NULL && !match->isType(Symbol::ID)){
      ident = match;
      chop(ident->token.size());
    }else{
      string s = *getToken();
      readNextToken();
      throw new InvalidTokenException(s);
    }
    break;
    
  case Tokenizer::EOL:
    ident = &Symbol::eol;
    unseparatedNumber = false;
    break;
    
  case Tokenizer::EOS:
    ident = &Symbol::eos;
    break;
    
  default:
    break;
  }
  return ident;
}

Symbol* Symbolizer::getSymbol(){
  return ident;
}
Identifier* Symbolizer::getIdentifier(){
  return ident;
}

/* 
   pre: token != NULL && symbol_table != NULL
   post: return waarde == Symbol || return waarde == NULL
*/
Symbol * Symbolizer::getMatch(string* token){
  string s(*token);
			
  for(string::size_type i = s.size(); i > 0; i--){
    s.erase(i);
    Symbol * symbol = symbol_table->get(s);
    if(symbol != NULL)
      return symbol;
  }
  return NULL;
}

Symbol* Symbolizer::symbolizeNumber(){
  if(lastRead == '.'){
    string s;
    while(lastRead == '.'){
      s.append(*getToken());
      readNextToken();
    }
    s.append(*getToken());
    readNextToken();
    throw new InvalidTokenException(s);
  }
  
  unseparatedNumber = (lastReadType != Tokenizer::WHITESPACE);
  return new Scalar(getToken()->c_str());
}
