#ifndef __Symbolizer_h__
#define __Symbolizer_h__

#include "Tokenizer.h"
#include "Symbol.h"
#include "SymbolTable.h"
#include <exception>

/*		Symbolizer (Extensie van Tokenizer)
		Leest een of meer tokens en herkent daarin een Symbol type. Tokens met het type NUMBER
		worden in een nieuw Constant instantie geplaatst. Bij andere tokens wordt getracht er 
		een Symbol uit de SymbolTable bij te zoeken. Als geen Symbol wordt gevonden wordt een 
		nieuwe Variable gemaakt en in de SymbolTable gezet.
*/

/*
  InvalidTokenExceptions worden gegeven bij tokens die niet in de SymbolTable
  staan en ook geen identifier kunnen zijn.
*/
class InvalidTokenException: public exception{
 public:
  char* err;
	
  InvalidTokenException(string& token){
    err = new char[16 + token.size()];
    int len = sprintf(err, "Invalid token: %s", token.c_str());
    err[len] = '\0';
  }
			
  virtual const char* what() const throw(){
    return err;
  }
};

typedef struct Symbol Identifier;

class Symbolizer: public Tokenizer{
 private:
  Identifier* ident;
  SymbolTable* symbol_table;
  bool unseparatedNumber;
		
  Symbol* symbolizeNumber();
  Symbol* getMatch(string * token);
		
 public:
  Symbolizer(SymbolTable* symbols, Reader * reader);

  void setSymbolTable(SymbolTable* symboltable);
  SymbolTable* getSymbolTable();
		
  Symbol* readNextSymbol() throw(InvalidTokenException*);
  Symbol* getSymbol();
  Identifier* getIdentifier();
};

#endif
