#include "Tokenizer.h"

Tokenizer::Tokenizer(Reader * reader){
  in = reader;
  lineNum = 1;
  position = 0;
  lastReadType = WHITESPACE;
  IDchar();
}

Tokenizer::~Tokenizer(){
  if(in != NULL){
    in->close();
  }
}

/* 
   pre: in != NULL
   post: lastRead != 0 && tokenType == 0-5
*/
void Tokenizer::IDchar(){
  if(lastReadType == EOS || ! in->read(lastRead))
    lastReadType = EOS;
  else if(lastRead == '\n')
    lastReadType = EOL;
  else if(isspace(lastRead))
    lastReadType = WHITESPACE;
  else if(isdigit(lastRead))
    lastReadType = NUMBER;
  else if(isalpha(lastRead))
    lastReadType = WORD;
  else
    lastReadType = UNKOWN;
}

/* 
   pre: in != NULL
   post: currentTokenType == 0-5 && 
   (currentToken == lastRead || currentToken == "end of line" || currentToken == "end of file")
*/				
int Tokenizer::readNextToken(){
  //skip whitespace and add length to position
  while(lastReadType == WHITESPACE){
    IDchar();
    position++;
  }
		
  //remove last token and add length to position
  position += currentToken.size();
  currentToken.clear();
  currentTokenType = lastReadType;
	
  switch(lastReadType){
		
  case UNKOWN:
    //check for numbers like ".5"
    if(lastRead == '.' && readNumber())
      currentTokenType = NUMBER;
    else{
      while(lastReadType == UNKOWN){
	currentToken.append(1, lastRead);
	IDchar();
      }
    }
    break;
		
  case WORD:
    while(lastReadType == WORD){
      currentToken.append(1, lastRead);
      IDchar();
    }
    break;
		
  case NUMBER:
    readNumber();
    break;
			
  case EOL:
    lineNum++;
    position = 0;
    //in plaats van IDchar zodat de volgende regel nog niet gelezen wordt
    lastReadType = WHITESPACE;
    break;
		
  case EOS:
    if(in != NULL){
      in->close();
      in = NULL;
    }
    break;
  default:
    break;
  }
  return currentTokenType;
}

string* Tokenizer::nextToken(){
  readNextToken();
  return getToken();
}

/* 
   pre: len > 0
   post: currentToken veranderd && currentTokenType veranderd
*/
void Tokenizer::chop(string::size_type len){
  if (len < currentToken.size()) {
    currentToken.erase(0, len);
    position += len;
		
    if (currentToken.size() == 1 && 
        currentToken.at(0) == '.' && 
        lastReadType == NUMBER) {
      //if punt: kijk of het een getal is
      
      while(lastReadType == NUMBER){
	currentToken.append(1, lastRead);
	IDchar();
      }
      currentTokenType = NUMBER;
    }
  } else
    readNextToken();
}

/* 
   pre: tokenType == 0-9 || tokenType == .
   post: currentToken bevat een getal
*/
bool Tokenizer::readNumber(){
  while(lastReadType == NUMBER){
    currentToken.append(1, lastRead);
    IDchar();
  }
  if(lastRead == '.'){
    currentToken.append(1, lastRead);
    IDchar();
		
    while(lastReadType == NUMBER){
      currentToken.append(1, lastRead);
      IDchar();
    }
  }
  return (currentToken.size() > 0);
}

string * Tokenizer::getToken(){
  return &currentToken;
}

Tokenizer::tokenType Tokenizer::getTokenType(){
  return currentTokenType;
}
int Tokenizer::getLineNumber(){
  return lineNum;
}
int Tokenizer::getPosition(){
  return position;
}
