#include <iostream>
#include <string>
#include "SymbolTable.h"
#include "Expression.h"

#include "FileReader.h"
#include "CinReader.h"
#include "FileWriter.h"

#include "MainLib.h"
#include "Parser.h"

#include "Executrix.h"

using namespace std;

/**
 * /main Interpreter
 * 
 */

FileWriter* writer;
Executrix* executrix;

void usage () {
  cout << "Voor text invoeren en uitvoer op scherm: " << endl;
  cout << "  Interpreter" << endl;
  cout << "Voor bestand invoeren en uitvoer op scherm: " << endl;
  cout << "  Interpreter infile.txt" << endl;
  cout << "Voor bestand invoeren en uitvoer naar bestand:" << endl;
  cout << "  Interpreter infile.txt > output.txt" << endl;
  cout << "Alternatief:" << endl;
  cout << "  cat infile.txt | Interpreter > output.txt" << endl;
}

/*	
	pre: msg wijst naar een array can len chars
	post: de tekst in msg wordt naar het bestand en het schem geschreven 
*/
void out(const char* msg, int len){
  writer->writeLine(msg, len);
  cout.write(msg, len);
}


/*
  pre: parser is geinitialiseerd
  post: geintepreteerde expressies worden naar out() geschreven samen met de
  uitkomst
*/
void processInput(Reader* reader, SymbolTable* symboltable){
  Symbolizer* symbolizer = new Symbolizer(symboltable, reader);
  Parser* parser = new Parser(symboltable, symbolizer);
  
  string str;
	
  while(1){
    Expression* exp = NULL;
    try{
      exp = parser->parseExpression();
      if(exp == NULL)
	break;
				
      //print expressie
      ostringstream stream1;
      stream1 << "Line " << exp->getLineNumber() << ": " << exp->toString() 
	      << endl;
      str = stream1.str();
      out(str.c_str(), str.size());
			
      //voer expressie uit
      Symbol* symbol = executrix->execute(exp);
			
      //print uitkomst
      str = string("> ");
      if(symbol == NULL)
	str.append("null");
      else
	str.append(symbol->getConstant()->toString());
      str.append("\n");
      out(str.c_str(), str.size());
			
      delete exp;
      exp = NULL;
    } catch(ParseException* e) {
      ostringstream stream1;
      stream1 << "Line " << symbolizer->getLineNumber() << ", Collumn " << 
	symbolizer->getPosition() << " Parse Error: " << e->what() << endl; 
      str = stream1.str();
      out(str.c_str(), str.size());
    } catch (RuntimeException* e) {
      ostringstream stream1;
      stream1 << "Runtime Error: " << e->what() << endl; 
      str = stream1.str();
      out(str.c_str(), str.size());
      delete exp;
    }
  }
  
  delete symbolizer;
  delete parser;
}

/*
  post: leest cin uit als er geen bestanden zijn opgegeven
  anders worden de bestanden geparsed
*/
int main(int argc, char * argv[]){
  string helpstr("-h");
  if (argc > 1 && helpstr.compare(argv[1]) == 0) {
    usage();
    return 0;
  }
  
  SymbolTable * symboltable = new SymbolTable();
  symboltable->setHardwired(new SymbolTable());
  MainLib::load(symboltable);
	
  executrix = new Executrix();
  writer = new FileWriter("output.txt");

  try {
    if(argc <= 1){
      cout << "For help run 'Interpreter -h'" << endl;
      Reader* reader = new CinReader(&cin);
      processInput(reader, symboltable);
      delete reader;
    } else {
      for (int i=1; i < argc; i++){
        Reader* reader = new FileReader(argv[i]);
        processInput(reader, symboltable);
        delete reader;
      }
    }
  } catch (IOException* e) {
    ostringstream stream1;
    string str;
    stream1 << " Error: " << e->what() << endl; 
    str = stream1.str();
    out(str.c_str(), str.size());
  }
		
  writer->close();
  delete writer;
  delete symboltable->getHardwired();
  delete symboltable;
  delete executrix;
  return 0;
}

